unit ChatServer;

interface

uses
  Classes, SysUtils, TextServer, UserInfo, ChatServerInterface;

type
  TDBAgent = class(TComponent)
  strict private
  public
    function LobbyUserCertify(UserInfo:TUserInfo):boolean; virtual; abstract;
    function RoomUserCertify(UserInfo:TUserInfo):boolean; virtual; abstract;
  end;

  TRoomServer = class(TChatServerInterface)
  strict private
  strict protected
    function ask_UseCertify(UserInfo:TUserInfo):boolean; override;
    procedure on_UserJoined(UserInfo:TUserInfo);
    procedure on_UserExited(UserInfo:TUserInfo);
    procedure on_RoomInfoChanged(Packet:TStringList);
  public
  end;

  TLobbyServer = class(TChatServerInterface)
  strict private
  strict protected
    function ask_UseCertify(UserInfo:TUserInfo):boolean; override;
  public
    procedure UserJoined(UserInfo:TUserInfo); virtual; abstract;
    procedure UserExited(UserInfo:TUserInfo); virtual; abstract;
    procedure RoomInfoChanged(Packet:TStringList); virtual; abstract;
  end;

  TChatServer = class(TComponent)
  strict private
    FRoomStartPort: integer;
    FRoomCount: integer;
    FLobbyPort: integer;
    FActive: boolean;
    procedure SetLobbyPort(const Value: integer);
    procedure SetRoomCount(const Value: integer);
    procedure SetRoomStartPort(const Value: integer);
  private
    FDBAgent : TDBAgent;
    FRoomList : TList;
    FLobbyServer : TLobbyServer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Open;
    procedure Close;
  published
    property Active : boolean read FActive;
    property LobbyPort : integer read FLobbyPort write SetLobbyPort;
    property RoomStartPort : integer read FRoomStartPort write SetRoomStartPort;
    property RoomCount : integer read FRoomCount write SetRoomCount;
  end;    

implementation

uses
  DBAgentImp, LobbyServerImp, RoomServerImp;

{ TChatServer }

constructor TChatServer.Create(AOwner: TComponent);
begin
  inherited;

  FActive := false;
  FLobbyPort := 2000;
  FRoomStartPort := 2001;
  FRoomCount := 10;

  FDBAgent := TDBAgentImp.Create(Self);
  FRoomList := TList.Create;
  FLobbyServer := TLobbyServerImp.Create(Self);
end;

destructor TChatServer.Destroy;
var
  Loop : integer;
  RoomServer : TRoomServer;
begin
  FLobbyServer.Free;

  for Loop := 0 to FRoomCount-1 do begin
    RoomServer := Pointer(FRoomList[Loop]);
    RoomServer.Close;
    RoomServer.Free;
  end;
  FRoomList.Free;

  inherited;
end;

procedure TChatServer.SetLobbyPort(const Value: integer);
begin
  if FActive = true then
    raise Exception.Create('TChatServer StartǾ Ӽ   ϴ.');

  FLobbyPort := Value;
end;

procedure TChatServer.SetRoomCount(const Value: integer);
begin
  if FActive = true then
    raise Exception.Create('TChatServer StartǾ Ӽ   ϴ.');

  FRoomCount := Value;
end;

procedure TChatServer.SetRoomStartPort(const Value: integer);
begin
  if FActive = true then
    raise Exception.Create('TChatServer StartǾ Ӽ   ϴ.');

  FRoomStartPort := Value;
end;

procedure TChatServer.Open;
var
  Loop : integer;
  RoomServer : TRoomServer;
begin
  FActive := true;

  FLobbyServer.Port := FLobbyPort;
  FLobbyServer.Open;

  for Loop := 0 to FRoomCount-1 do begin
    RoomServer := TRoomServerImp.Create(Self);
    RoomServer.Port := FRoomStartPort + Loop;
    RoomServer.Open;

    FRoomList.Add(RoomServer);
  end;
end;

procedure TChatServer.Close;
var
  Loop : integer;
  RoomServer : TRoomServer;
begin
  FLobbyServer.Close;

  for Loop := 0 to FRoomCount-1 do begin
    RoomServer := Pointer(FRoomList[Loop]);
    RoomServer.Close;
    RoomServer.Free;
  end;
  FRoomList.Clear;

  FActive := false;
end;

{ TRoomServer }

function TRoomServer.ask_UseCertify(UserInfo: TUserInfo): boolean;
begin
  Result := TChatServer(Owner).FDBAgent.RoomUserCertify(UserInfo);
end;

procedure TRoomServer.on_RoomInfoChanged(Packet: TStringList);
begin
  TChatServer(Owner).FLobbyServer.RoomInfoChanged(Packet);
end;

procedure TRoomServer.on_UserExited(UserInfo: TUserInfo);
begin
  TChatServer(Owner).FLobbyServer.UserExited(UserInfo);
end;

procedure TRoomServer.on_UserJoined(UserInfo: TUserInfo);
begin
  TChatServer(Owner).FLobbyServer.UserJoined(UserInfo);
end;

{ TLobbyServer }

function TLobbyServer.ask_UseCertify(UserInfo: TUserInfo): boolean;
begin
  Result := TChatServer(Owner).FDBAgent.LobbyUserCertify(UserInfo);
end;

end.
