unit ChatServerInterface;

interface

uses
  Classes, SysUtils, TextServer, UserInfo, ScktComp;

type
  TChatServerInterface = class(TTextServer)
  private
    procedure on_Connected(Connection:TObject; UserInfo:TUserInfo); override;
    procedure on_BeforeDisconnected(Connection:TObject; UserInfo:TUserInfo); override;
    procedure on_AfterDisconnected(Connection:TObject; UserInfo:TUserInfo); override;
    procedure on_ReceivePacket(Connection:TObject; UserInfo:TUserInfo; Packet:TStringList); override;
  protected
    function ask_UseCertify(UserInfo:TUserInfo):boolean; virtual; abstract;
  public
    procedure sp_ErLogin(UserInfo:TUserInfo);
    procedure sp_IDinUse(UserInfo:TUserInfo);
    procedure sp_OkLogin(UserInfo:TUserInfo);
    procedure sp_UserIn(UserInfo:TUserInfo);
    procedure sp_UserOut(UserInfo:TUserInfo);
    procedure sp_UserList(UserInfo:TUserInfo; UserList:string);
  published
    procedure rp_Login(Connection:TObject; UserInfo:TUserInfo; Packet:TStringList);
    procedure rp_Logout(Connection:TObject; UserInfo:TUserInfo; Packet:TStringList);
    procedure rp_AskUserList(Connection:TObject; UserInfo:TUserInfo; Packet:TStringList);
  end;

implementation

{ TRoomServerImp }

procedure TChatServerInterface.on_AfterDisconnected(Connection: TObject;
  UserInfo: TUserInfo);
begin
  sp_UserOut(UserInfo);
end;

procedure TChatServerInterface.on_BeforeDisconnected(Connection: TObject;
  UserInfo: TUserInfo);
begin
  // Todo:
end;

procedure TChatServerInterface.on_Connected(Connection: TObject; UserInfo: TUserInfo);
begin
  // Flex Croess Domain  ذ 
  SendText(
    UserInfo,
    '<cross-domain-policy>' +
    '  <allow-access-from domain="*" to-ports="*" />' +
    '</cross-domain-policy>' + #10 + #0,
    true
  );
end;

procedure TChatServerInterface.on_ReceivePacket(Connection: TObject;
  UserInfo: TUserInfo; Packet: TStringList);
var
  Proc : procedure (Connection:TObject; UserInfo:TUserInfo; Packet:TStringList) of object;
begin
  TMethod(Proc).Data := Self;
  TMethod(Proc).Code := MethodAddress('rp_' + Packet.Values['Code']);
  if Assigned(Proc) then Proc(Connection, UserInfo, Packet);
end;

procedure TChatServerInterface.rp_AskUserList(Connection: TObject;
  UserInfo: TUserInfo; Packet: TStringList);
var
  List : TList;
  Loop : integer;
  Item : TUserInfo;
  stUserList : string;
begin
  stUserList:= '';

  List := LockList;
  try
  for Loop := 0 to List.Count - 1 do begin
    Item := GetUserInfo(List[Loop]);
    if Item = nil then Continue;
    if Item.Logined = false then Continue;

    stUserList:= stUserList + Item.Text + #10'end.'#10;
  end;
  finally
    UnlockList;
  end;

  sp_UserList(UserInfo, stUserList);
end;

procedure TChatServerInterface.rp_Login(Connection: TObject; UserInfo: TUserInfo;
  Packet: TStringList);
var
  List : TList;
  ExUser : TUserInfo;
begin
  if UserInfo.Logined then Exit;

  UserInfo.UserID :=   Packet.Values['UserID'];
  UserInfo.Password := Packet.Values['Password'];

  if ask_UseCertify(UserInfo) = false then begin
    sp_ErLogin(UserInfo);
    Exit;
  end;

  List := LockList;
  try
    ExUser:= FindUserID(UserInfo.UserID, List);
    if ExUser <> nil then begin
      ExUser.Logined := false;
      sp_IDinUse(ExUser);
      CloseConnection(Connection);
    end;
  finally
    UnlockList;
  end;

  UserInfo.Logined := true;

  sp_OkLogin(UserInfo);
  sp_UserIn(UserInfo);
end;

procedure TChatServerInterface.rp_Logout(Connection: TObject; UserInfo: TUserInfo;
  Packet: TStringList);
begin
  CloseConnection(Connection);
end;

procedure TChatServerInterface.sp_ErLogin(UserInfo: TUserInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Values['Code'] := 'ErLogin';

    SendText(UserInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TChatServerInterface.sp_IDinUse(UserInfo: TUserInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Values['Code'] := 'IDinUse';

    SendText(UserInfo, Packet.Text, true);
  finally
    Packet.Free;
  end;
end;

procedure TChatServerInterface.sp_OkLogin(UserInfo: TUserInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Text := UserInfo.Text;
    Packet.Values['Code'] := 'OkLogin';

    SendText(UserInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TChatServerInterface.sp_UserIn(UserInfo: TUserInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Text := UserInfo.Text;
    Packet.Values['Code'] := 'UserIn';

    SendToOther(UserInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TChatServerInterface.sp_UserList(UserInfo: TUserInfo; UserList: string);
var
  Packet : TStringList;
begin
  Packet:= TStringList.Create;
  try
    Packet.Values['Code'] := 'UserList';
    Packet.Add(UserList);

    SendText(UserInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TChatServerInterface.sp_UserOut(UserInfo: TUserInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Values['Code'] := 'UserOut';
    Packet.Values['UserID'] := UserInfo.UserID;

    SendToOther(UserInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

end.
