unit _fmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ScktComp, EasyServerSocket, StdCtrls, EasySocketInfo, ExtCtrls;

type
  TfmMain = class(TForm)
    moMsg: TMemo;
    Timer: TTimer;
    ServerSocket: TEasyServerSocket;
    procedure ServerSocketClientConnected(Sender: TObject;
      ASocket: TCustomWinSocket; SocketInfo: TEasySocketInfo);
    procedure ServerSocketClientDisconnected(Sender: TObject;
      ASocket: TCustomWinSocket; SocketInfo: TEasySocketInfo);
    procedure TimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ServerSocketReceivePacket(Sender: TObject;
      ASocket: TCustomWinSocket; SocketInfo: TEasySocketInfo;
      Packet: TStringList);
  private
  public
    procedure sp_ErLogin(SocketInfo:TEasySocketInfo);
    procedure sp_IDinUse(SocketInfo:TEasySocketInfo);
    procedure sp_OkLogin(SocketInfo:TEasySocketInfo);
    procedure sp_UserIn(SocketInfo:TEasySocketInfo);
    procedure sp_UserOut(SocketInfo:TEasySocketInfo);
    procedure sp_UserList(SocketInfo:TEasySocketInfo; UserList:string);
  published
    procedure rp_Login(ASocket:TCustomWinSocket; SocketInfo:TEasySocketInfo; Packet:TStringList);
    procedure rp_Logout(ASocket:TCustomWinSocket; SocketInfo:TEasySocketInfo; Packet:TStringList);
    procedure rp_AskUserList(ASocket:TCustomWinSocket; SocketInfo:TEasySocketInfo; Packet:TStringList);
  end;

var
  fmMain: TfmMain;

implementation

{$R *.dfm}

procedure TfmMain.FormCreate(Sender: TObject);
begin
  ServerSocket.Open;
end;

procedure TfmMain.FormDestroy(Sender: TObject);
begin
  ServerSocket.Close;
end;

procedure TfmMain.rp_AskUserList(ASocket: TCustomWinSocket;
  SocketInfo: TEasySocketInfo; Packet: TStringList);
var
  Loop : integer;
  stUserList : string;
  UserInfo : TEasySocketInfo;
begin
  stUserList:= '';

  for Loop := 0 to ServerSocket.ActiveConnections - 1 do begin
    UserInfo:= Pointer(ServerSocket.Connections[Loop].Data);
    if UserInfo = nil then Continue;
    if UserInfo.Logined = false then Continue;

    stUserList:= stUserList + UserInfo.Text + #10'end.'#10;
  end;

  sp_UserList(SocketInfo, stUserList);
end;

procedure TfmMain.rp_Login(ASocket: TCustomWinSocket;
  SocketInfo: TEasySocketInfo; Packet: TStringList);
var
  ExUser : TEasySocketInfo;
begin
  if SocketInfo.Logined then Exit;

  SocketInfo.UserID :=   Packet.Values['UserID'];
  SocketInfo.Password := Packet.Values['Password'];

  // Todo:
//  if UserCertify(UserInfo) = false then begin
//    sp_ErLogin(Connection);
//    Exit;
//  end;

  ExUser:= ServerSocket.FindUserByUserID(SocketInfo.UserID);
  if ExUser <> nil then begin
    ExUser.Logined := false;
    sp_IDinUse(ExUser);
    ExUser.Socket.Close;
  end;

  SocketInfo.Logined := true;

  sp_OkLogin(SocketInfo);
  sp_UserIn(SocketInfo);
end;

procedure TfmMain.rp_Logout(ASocket: TCustomWinSocket;
  SocketInfo: TEasySocketInfo; Packet: TStringList);
begin
  ASocket.Close;
end;

procedure TfmMain.ServerSocketClientConnected(Sender: TObject;
  ASocket: TCustomWinSocket; SocketInfo: TEasySocketInfo);
begin
  // Flex Croess Domain  ذ 
  ASocket.SendText(
    '<cross-domain-policy>' +
    '  <allow-access-from domain="*" to-ports="*" />' +
    '</cross-domain-policy>' + #10 + #0
  );
end;

procedure TfmMain.ServerSocketClientDisconnected(Sender: TObject;
  ASocket: TCustomWinSocket; SocketInfo: TEasySocketInfo);
begin
  sp_UserOut(SocketInfo);
end;

procedure TfmMain.ServerSocketReceivePacket(Sender: TObject;
  ASocket: TCustomWinSocket; SocketInfo: TEasySocketInfo; Packet: TStringList);
var
  Proc : procedure (ASocket:TCustomWinSocket; SocketInfo:TEasySocketInfo; Packet:TStringList) of object;
begin
  Packet.Text := StringReplace(Packet.Text, #10, #13#10, [rfReplaceAll]);

//  moMsg.Lines.Add(Text);
//  moMsg.Lines.Add('');

  TMethod(Proc).Data := Self;
  TMethod(Proc).Code := MethodAddress('rp_' + Packet.Values['Code']);
  if Assigned(Proc) then Proc(ASocket, SocketInfo, Packet);
end;

procedure TfmMain.sp_ErLogin(SocketInfo: TEasySocketInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Values['Code'] := 'ErLogin';

    ServerSocket.SendTo(SocketInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TfmMain.sp_IDinUse(SocketInfo: TEasySocketInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Values['Code'] := 'IDinUse';

    ServerSocket.SendTo(SocketInfo, Packet.Text);
    SocketInfo.SendNow;
  finally
    Packet.Free;
  end;
end;

procedure TfmMain.sp_OkLogin(SocketInfo: TEasySocketInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Text := SocketInfo.Text;
    Packet.Values['Code'] := 'OkLogin';

    ServerSocket.SendTo(SocketInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TfmMain.sp_UserIn(SocketInfo: TEasySocketInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Text := SocketInfo.Text;
    Packet.Values['Code'] := 'UserIn';

    ServerSocket.SendToOther(SocketInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TfmMain.sp_UserList(SocketInfo: TEasySocketInfo; UserList: string);
var
  Packet : TStringList;
begin
  Packet:= TStringList.Create;
  try
    Packet.Values['Code'] := 'UserList';
    Packet.Add(UserList);

    ServerSocket.SendTo(SocketInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TfmMain.sp_UserOut(SocketInfo: TEasySocketInfo);
var
  Packet : TStringList;
begin
  Packet := TStringList.Create;
  try
    Packet.Values['Code'] := 'UserOut';
    Packet.Values['UserID'] := SocketInfo.UserID;

    ServerSocket.SendToOther(SocketInfo, Packet.Text);
  finally
    Packet.Free;
  end;
end;

procedure TfmMain.TimerTimer(Sender: TObject);
var
  Loop : integer;
  Item : TEasySocketInfo;
begin
  for Loop := ServerSocket.ActiveConnections - 1 downto 0 do begin
    Item:= Pointer(ServerSocket.Connections[Loop].Data);
    if Item = nil then Continue;
    
    Item.CheckTickCount;
  end;
end;

end.

